@@ -46,18 +46,21 @@ sub parse_arg {
     }
 
     my $target1 = $self->arg_co_maybe (@arg ? $arg[0] : '');
+    my $target2;
 
     if ($self->{from}) {
         # When using "from", $target1 must always be a depotpath.
         if (defined $target1->{copath}) {
-            # Because merging under the copath anchor is unsafe,
-            # cast it into a coroot now. -- XXX -- also see Update.pm
-            my $entry = $self->{xd}{checkout}->get ($target1->{copath});
-            $target1 = $self->arg_depotpath ($entry->{depotpath});
+            # Because merging under the copy anchor is unsafe, we always merge
+            # to the most immediate copy anchor under copath root.
+            ($target1, $target2) = $self->find_checkout_anchor (
+                $target1, 1, $self->{sync}
+            );
+            delete $target1->{copath};
         }
     }
 
-    my $target2 = $target1->copied_from($self->{sync});
+    $target2 ||= $target1->copied_from($self->{sync});
     if (!defined ($target2)) {
         die loc ("Cannot find the path which '%1' copied from.\n", $arg[0]);
     }
@@ -85,6 +88,12 @@ sub run {
     my $fs = $repos->fs;
     my $yrev = $fs->youngest_rev;
 
+    if (my @mirrors = $dst->contains_mirror) {
+	die loc ("%1 can not be used as merge target, because it contains mirrored path:\n", $dst->{report})
+	    .join("\n", @mirrors, '')
+		unless $mirrors[0] eq $dst->path;
+    }
+
     if ($self->{sync}) {
         my $sync = $self->command ('sync');
 	my (undef, $m) = resolve_svm_source($repos, find_svm_source($repos, $src->{path}));
@@ -116,7 +125,13 @@ sub run {
 	my ($baserev, $torev) = @{$revlist[0]};
 	$merge = SVK::Merge->new
 	    (%$self, repos => $repos, src => $src->new (revision => $torev),
-	     dst => $dst, base => $src->new (revision => $baserev), target => '');
+	     dst => $dst, base => $src->new (revision => $baserev), target => '',
+	     fromrev => $baserev);
+    }
+
+    if ($merge->{fromrev} == $merge->{src}{revision}) {
+	print loc ("Empty merge.\n");
+	return;
     }
 
     $self->get_commit_message ($self->{log} ? $merge->log(1) : '')
@@ -144,13 +159,20 @@ sub run {
         );
 
 	my $spool = SVN::Pool->new_default;
-	for (@rev) {
-	    $merge = SVK::Merge->auto (%$merge, src => $src->new (revision => $_));
+	my $previous_base;
+	foreach my $rev (@rev) {
+	    $merge = SVK::Merge->auto (%$merge, src => $src->new (revision => $rev));
+	    if ($previous_base) {
+		$merge->{fromrev} = $previous_base;
+	    }
+
 	    print '===> '.$merge->info;
 	    $self->{message} = $merge->log (1);
+
 	    last if $merge->run ($self->get_editor ($dst));
 	    # refresh dst
 	    $dst->{revision} = $fs->youngest_rev;
+	    $previous_base = $rev;
 	    $spool->clear;
 	}
     }
@@ -187,6 +209,7 @@ SVK::Command::Merge - Apply differences between two sources
  -s [--sync]            : synchronize mirrored sources before update
  -t [--to]              : merge to the specified path
  -f [--from]            : merge from the specified path
+ -P [--patch] arg       : instead of commit, save this change as a patch
  -S [--sign]            : sign this change
  --no-ticket            : do not record this merge point
  --track-rename         : track changes made to renamed node
