@@ -76,11 +76,16 @@ sub node_status {
     return $self->{status}{$path}[0];
 }
 
+my %prop = ( 'U' => 0, 'g' => 1, 'G' => 2, 'M' => 3, 'C' => 4);
+
 sub prop_status {
     my ($self, $path, $s) = @_;
     my $st = $self->{status}{$path};
     $st->[1] = $s if defined $s
-	&& !($st->[0] && ($st->[0] eq 'A' || $st->[0] eq 'R'));
+	# node status allow prop
+	&& !($st->[0] && ($st->[0] eq 'A' || $st->[0] eq 'R'))
+	    # not overriding things more informative
+	    && (!$st->[1] || $prop{$s} > $prop{$st->[1]});
     return $self->{status}{$path}[1];
 }
 
@@ -92,6 +97,7 @@ sub hist_status {
 
 sub flush {
     my ($self, $path, $anchor) = @_;
+    return if $self->{quiet};
     my $status = $self->{status}{$path};
     if ($status && grep {$_} @{$status}[0..2]) {
 	$self->{cb_flush}->($path, $status) if $self->{cb_flush};
@@ -104,11 +110,12 @@ sub flush {
 
 sub flush_dir {
     my ($self, $path) = @_;
+    return if $self->{quiet};
     for (grep {$path ? index($_, "$path/") == 0 : $_}
 	 sort keys %{$self->{status}}) {
 	$self->flush ($_, $path eq $_);
     }
-    $self->flush ($path, 1) unless $path;
+    $self->flush ($path, 1);
 }
 
 =head1 AUTHORS
