@@ -156,8 +156,14 @@ sub descend {
     my ($self, $entry) = @_;
     $self->{depotpath} .= "/$entry";
     $self->{path} .= "/$entry";
-    $self->{report} = catfile ($self->{report}, $entry);
-    $self->{copath} = catfile ($self->{copath}, $entry);
+
+    if (defined $self->{copath}) {
+        $self->{report} = catfile ($self->{report}, $entry);
+        $self->{copath} = catfile ($self->{copath}, $entry);
+    }
+    else {
+        $self->{report} = "$self->{report}/$entry";
+    }
 }
 
 =head2 universal
@@ -180,6 +186,15 @@ sub contains_copath {
     return 0;
 }
 
+sub contains_mirror {
+    require SVN::Mirror;
+    my ($self) = @_;
+    my $path = $self->{path};
+    $path .= '/' unless $path eq '/';
+    return map { substr ("$_/", 0, length($path)) eq $path ? $_ : () }
+	SVN::Mirror::list_mirror ($self->{repos});
+}
+
 sub depotname {
     my $self = shift;
 
